﻿using Microsoft.EntityFrameworkCore;

namespace Netina.Repository.Handlers.Products;

public class ChangeProductDisplayedCommandHandler(IRepositoryWrapper repositoryWrapper)
    : IRequestHandler<ChangeProductDisplayedCommand, bool>
{
    public async Task<bool> Handle(ChangeProductDisplayedCommand request, CancellationToken cancellationToken)
    {
        var ent = await repositoryWrapper.SetRepository<Product>()
            .TableNoTracking
            .FirstOrDefaultAsync(p => p.Id == request.Id, cancellationToken);
        if (ent == null)
            throw new AppException("Product not found");
        var newEnt = Product.Create(ent.PersianName, ent.EnglishName, ent.Summery, ent.ExpertCheck, ent.Tags,
            ent.Warranty,
            request.BeDisplayed, ent.Cost, ent.PackingCost, ent.HasExpressDelivery, ent.Stock, ent.MaxOrderCount,
            ent.BrandId,
            ent.CategoryId);
        newEnt.CreatedAt = ent.CreatedAt;
        newEnt.CreatedBy = ent.CreatedBy;
        newEnt.Id = ent.Id;

        repositoryWrapper.SetRepository<Product>()
            .Update(newEnt);

        await repositoryWrapper.SaveChangesAsync(cancellationToken);
        return true;
    }
}